@@ -7,13 +7,13 @@ for which a new license (GPL+exception) is in place.
 #ifndef SCRIBUS_PAGEITEMATTRIBUTES_H
 #define SCRIBUS_PAGEITEMATTRIBUTES_H
 
-#include "pageitemattributesbase.h"
+#include "ui_pageitemattributes.h"
 
-class PageItemAttributes : public PageItemAttributesBase
+class PageItemAttributes : public QDialog, Ui::PageItemAttributes
 {
     Q_OBJECT
 	public:
-		PageItemAttributes( QWidget* parent = 0, const char* name = 0, bool modal = false, WFlags fl = 0 );
+		PageItemAttributes( QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0 );
 		~PageItemAttributes();
 
 		virtual ObjAttrVector * getNewAttributes();
@@ -32,6 +32,13 @@ class PageItemAttributes : public PageItemAttributesBase
 		virtual void clearEntries();
 		virtual void copyEntry();
 		virtual void okClicked();
+
+	private:
+		ObjAttrVector localAttributes;
+		ObjAttrVector localDocAttributes;
+		QStringList relationships;
+		QStringList relationshipsData;
+		QStringList nameList;
 };
 
 #endif
