@@ -26,41 +26,40 @@ for which a new license (GPL+exception) is in place.
 #include "scconfig.h"
 #endif
 // include files for QT
-#include <qobject.h>
-#include <qdict.h>
-#include <qcolor.h>
-#include <qvaluelist.h>
-#include <qvaluevector.h>
-#include <qpixmap.h>
-#include <qptrlist.h>
-#include <qfont.h>
-#include <qmap.h>
-#include <qpointarray.h>
-#include <qstringlist.h>
-#include <qtimer.h>
+#include <QColor>
+#include <QFont>
+#include <QList>
+#include <QMap>
+#include <QObject>
+#include <QPixmap>
+#include <QRectF>
+#include <QStringList>
+#include <QTimer>
+
 
 #include "gtgettext.h" //CB For the ImportSetup struct and itemadduserframe
 #include "scribusapi.h"
-#include "scribusstructs.h"
-#include "pagesize.h"
+#include "undostate.h"
+#include "observable.h"
 #include "prefsstructs.h"
 #include "documentinformation.h"
 #include "undoobject.h"
 #include "page.h"
 #include "pageitem.h"
-#include "pageitem_line.h"
-#include "pageitem_textframe.h"
 #include "pagestructs.h"
 #include "usertaskstructs.h"
 #include "styles/styleset.h"
-#include "scpattern.h"
 #include "scguardedptr.h"
+#include "updatemanager.h"
+#include "sclayer.h"
 
-#include CMS_INC
+#include "colormgmt/sccolorprofile.h"
+#include "colormgmt/sccolortransform.h"
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
+class DocUpdater;
 class UndoManager;
 class UndoState;
 class PDFOptions;
@@ -69,13 +68,50 @@ class Selection;
 class ScribusView;
 class ScribusMainWindow;
 class ResourceCollection;
+class PageSize;
+class ScPattern;
+class UndoTransaction;
 
 class QProgressBar;
 
+struct SCRIBUS_API NodeEditContext : public MassObservable<QPointF>
+{
+	enum SubMode { MOVE_POINT = 0, ADD_POINT = 1, DEL_POINT = 2, SPLIT_PATH = 3 };
+	int submode;
+	bool isContourLine;
+	FPointArray *oldClip;
+	UndoTransaction* nodeTransaction;
+	double oldItemX;
+	double oldItemY;
+		
+	int ClRe;
+	int ClRe2;
+	int SegP1;
+	int SegP2;
+	bool EdPoints;
+	bool MoveSym;
+	QList<int> SelNode;	
+	
+	NodeEditContext();
+	
+	bool hasNodeSelected();
+	void deselect();
+	
+	void reset();
+	
+	void reset1Control(PageItem* currItem);
+	void resetControl(PageItem* currItem);
+	FPointArray beginTransaction(PageItem* currItem);
+	void finishTransaction(PageItem* currItem);
+	ItemState<QPair<FPointArray, FPointArray> >* finishTransaction1(PageItem* currItem);
+	void finishTransaction2(PageItem* currItem, ItemState<QPair<FPointArray, FPointArray> >* state);
+	void moveClipPoint(PageItem *currItem, FPoint ip);
+};
+
 
 /**! \brief the Document Class
   */
-class SCRIBUS_API ScribusDoc : public QObject, public UndoObject
+class SCRIBUS_API ScribusDoc : public QObject, public UndoObject, public Observable<ScribusDoc>
 {
 	Q_OBJECT
 
@@ -91,7 +127,7 @@ public:
 	bool isModified() const;
 /** Setzt die Seitenattribute */
 	void setPage(double b, double h, double t, double l, double r, double bo, double sp, double ab, bool atf, int fp);
-	void resetPage(double t, double l, double r, double bo, int fp);
+	void resetPage(MarginStruct& newMargins, int fp);
 
 	/**
 	 * @brief Return the view associated with the document
@@ -99,12 +135,22 @@ public:
 	ScribusView* view() const;
 	ScribusMainWindow* scMW() const {return m_ScMW;}
 	void setGUI(bool hasgui, ScribusMainWindow* mw, ScribusView* view);
+	void createHyphenator();
 
 	/**
 	 * @brief Return the guarded object associated with the document
 	 */
 	const ScGuardedPtr<ScribusDoc>& guardedPtr() const;
 	
+	UpdateManager* updateManager() { return &m_updateManager; }
+	MassObservable<PageItem*> * itemsChanged() { return &m_itemsChanged; }
+	MassObservable<Page*>     * pagesChanged() { return &m_pagesChanged; }
+	MassObservable<QRectF>    * regionsChanged() { return &m_regionsChanged; }
+	
+	void invalidateAll();
+	void invalidateLayer(int layerID);
+	void invalidateRegion(QRectF region);
+	
 	// Add, delete and move pages
 	
 	Page* addPage(const int pageNumber, const QString& masterPageName=QString::null, const bool addAutoFrame=false);
@@ -112,6 +158,10 @@ public:
 	//! @brief Add a master page with this function, do not use addPage
 	Page* addMasterPage(const int, const QString&);
 	void deleteMasterPage(const int);
+	//! @brief Rebuild master name list
+	void rebuildMasterNames(void);
+	//! @brief Replace a master page by default one
+	void replaceMasterPage(const QString& oldMasterPage);
 	//! @brief Rename a master page
 	bool renameMasterPage(const QString& oldPageName, const QString& newPageName);
 	//! @brief Create the default master pages based on the layout selected by the user, ie, Normal, Normal Left, etc.
@@ -131,12 +181,12 @@ public:
 	void setLocationBasedPageLRMargins(uint pageIndex);
 	/**
 	 * @brief Move page(s) within the document
-	 * @param from page index
-	 * @param to page index
-	 * @param ziel target to move to (page index)
-	 * @param art Before, After or at the end
+	 * @param fromPage page index
+	 * @param toPage page index
+	 * @param count target to move to (page index)
+	 * @param position Before, After or at the end
 	 */
-	void movePage(const int from, const int to, const int ziel, const int art);
+	void movePage(const int fromPage, const int toPage, const int dest, const int position);
 	
 	/**
 	 * @brief Copy a page (pageNumberToCopy) copyCount times, whereToInsert(before or after) the existingPage or at the end.
@@ -221,10 +271,10 @@ public:
 	/**
 	 * @brief Set the layer locked via the layer number
 	 * @param layerNumber Number of the layer
-	 * @param isViewable true = layer is locked
+	 * @param isLocked true = layer is locked
 	 * @return Success or failure
 	 */
-	bool setLayerLocked(const int layerNumber, const bool isViewable);
+	bool setLayerLocked(const int layerNumber, const bool isLocked);
 	/**
 	 * @brief Is the layer locked
 	 * @param layerNumber Number of the layer
@@ -260,7 +310,7 @@ public:
 	/**
 	 * @brief Set the layer layerBlendMode via the layer number
 	 * @param layerNumber Number of the layer
-	 * @param trans layerBlendMode
+	 * @param blend layerBlendMode
 	 * @return Success or failure
 	 */
 	bool setLayerBlendMode(const int layerNumber, int blend);
@@ -342,7 +392,7 @@ public:
 	 * @param layerNumber Number of the layer
 	 * @return Name of the layer
 	 */
-	const QString &layerName(const int layerNumber) const;
+	QString layerName(const int layerNumber) const;
 	/**
 	 * @brief Change the name of a layer
 	 * @param layerNumber number of the layer
@@ -377,20 +427,25 @@ public:
 	void getUsedProfiles(ProfilesL& usedProfiles);
 	bool OpenCMSProfiles(ProfilesL InPo, ProfilesL InPoCMYK, ProfilesL MoPo, ProfilesL PrPo);
 	void CloseCMSProfiles();
+	void SetDefaultCMSParams();
 	/**
 	 * @brief Switch Colormanagement on or of
 	 * @param enable bool, if true Colormanagement is switched on, else off
 	 */
 	void enableCMS(bool enable);
 	
-	const ParagraphStyle& paragraphStyle(QString name) { return docParagraphStyles[name]; }
+	const ParagraphStyle& paragraphStyle(QString name) { return docParagraphStyles.get(name); }
 	const StyleSet<ParagraphStyle>& paragraphStyles()   { return docParagraphStyles; }
 	bool isDefaultStyle( const ParagraphStyle& p ) const { return docParagraphStyles.isDefault(p); }
 	bool isDefaultStyle( const CharStyle& c ) const { return docCharStyles.isDefault(c); }
 // 	bool isDefaultStyle( LineStyle& l ) const { return MLineStyles......; }
 
 	void getNamedResources(ResourceCollection& lists) const;
-	void replaceNamedResources(ResourceCollection& newNames);	
+	void replaceNamedResources(ResourceCollection& newNames);
+	bool styleExists(QString styleName);
+
+	QList<int> getSortedStyleList();
+	QList<int> getSortedCharStyleList();
 	
 	void redefineStyles(const StyleSet<ParagraphStyle>& newStyles, bool removeUnused=false);
 	/**
@@ -399,11 +454,25 @@ public:
 	 * @param newNameForOld a map which maps the name of any style to remove to a new stylename
 	 */
 	void replaceStyles(const QMap<QString,QString>& newNameForOld);
-	void loadStylesFromFile(QString fileName, StyleSet<ParagraphStyle> *tempStyles = NULL,
-	                                          StyleSet<CharStyle> *tempCharStyles = NULL,
-	                                          QMap<QString, multiLine> *tempLineStyles = NULL);
+	/**
+	 * @brief Insert styles from another document in this document.
+	 *        
+	 * @param fileName The path of the document we want to extract its styles
+	 */
+	void loadStylesFromFile(QString fileName);
+	/**
+	 * @brief Gather styles from another document.
+	 *        
+	 * @param fileName The path of the document we want to extract its styles
+	 * @param tempStyles A pointer to a StyleSet which will be filled by paragraph styles
+	 * @param tempCharStyles A pointer to a StyleSet which will be filled by character styles
+	 * @param tempLineStyles A map which will be filled by line styles
+	 */
+	void loadStylesFromFile(QString fileName, StyleSet<ParagraphStyle> *tempStyles,
+	                                          StyleSet<CharStyle> *tempCharStyles,
+	                                          QMap<QString, multiLine> *tempLineStyles);
 
-	const CharStyle& charStyle(QString name) { return docCharStyles[name]; }
+	const CharStyle& charStyle(QString name) { return docCharStyles.get(name); }
 	const StyleSet<CharStyle>& charStyles()  { return docCharStyles; }
 	void redefineCharStyles(const StyleSet<CharStyle>& newStyles, bool removeUnused=false);
 	/**
@@ -440,14 +509,20 @@ public:
 
 	bool AddFont(QString name, int fsize = 10);
 	/*!
-	 * @brief Replace line style colors
+	 * @brief TODO: Reorganise the fonts.. how? Moved from scribus.cpp
+	 * CB: almost the same as getUsedFonts???
 	 */
-	void replaceLineStyleColors(const QMap<QString, QString>& colorMap);
+	QMap<QString,int> reorganiseFonts();
 	/*!
 	 * @brief Returns a qmap of the fonts and  their glyphs used within the document
 	 */
 	void getUsedFonts(QMap<QString,QMap<uint, FPointArray> > &Really);
 	void checkItemForFonts(PageItem *it, QMap<QString, QMap<uint, FPointArray> > & Really, uint lc);
+
+	/*!
+	 * @brief Replace line style colors
+	 */
+	void replaceLineStyleColors(const QMap<QString, QString>& colorMap);
 	/*!
 	* @brief Builds a qmap of the colours used within the document
 	*/
@@ -465,12 +540,8 @@ public:
 	* @brief Builds a QStringList of the patterns used within the document
 	*/
 	QStringList getUsedPatterns();
-	QStringList getUsedPatternsSelection();
+	QStringList getUsedPatternsSelection(Selection* customSelection);
 	QStringList getUsedPatternsHelper(QString pattern, QStringList &results);
-	/*!
-	 * @brief TODO: Reorganise the fonts.. how? Moved from scribus.cpp
-	 */
-	void reorganiseFonts();
 	/**
 	 * @brief Set and get the document's unit index
 	 */
@@ -485,15 +556,21 @@ public:
 	 * @brief Undo function for applying a master page
 	 */
 	void restoreMasterPageApplying(SimpleState *state, bool isUndo);
-	void restorePageCopy(SimpleState *state, bool isUndo);
+	void restoreCopyPage(SimpleState *state, bool isUndo);
+	void restoreMovePage(SimpleState *state, bool isUndo);
+	/**
+	 * @brief Undo function for grouping/ungrouping
+	 */
+	void restoreGrouping(SimpleState *state, bool isUndo);
+	void restoreUngrouping(SimpleState *state, bool isUndo);
 	/**
 	 * @brief Save function
 	 */
-	bool save(const QString&);
+	bool save(const QString& fileName, QString* savedFile = NULL);
 	/**
 	 * @brief Set the page margins. Current code uses current page only, also provide a (currently, TODO) option for this.
 	 */
-	bool changePageMargins(const double initialTop, const double initialBottom, const double initialLeft, const double initialRight, const double initialHeight, const double initialWidth, const double Height, const double width, const int orientation, const QString& pageSize, const int pageNumber=-1, const int pageType = 0);
+	bool changePageMargins(const double initialTop, const double initialBottom, const double initialLeft, const double initialRight, const double initialHeight, const double initialWidth, const double Height, const double width, const int orientation, const QString& pageSize, const int marginPreset, const bool moveObjects, const int pageNumber=-1, const int pageType = 0);
 	/**
 	 * @brief Recalculate the colors after CMS settings change. Update the items in the doc accordingly.
 	 */
@@ -505,7 +582,7 @@ public:
 	/**
 	 * @brief Copies a normal page to be a master pages
 	 */
-	const bool copyPageToMasterPage(const int, const int, const int, const QString&);
+	bool copyPageToMasterPage(const int, const int, const int, const QString&, bool);
 	/**
 	 * @brief Paste an item to the document.
 	 * The bulk of a paste item process runs here for want of a better place, but its a better place
@@ -539,19 +616,8 @@ public:
 	
 	/**
 	 * @brief Allow the user to create a frame easily with some simple placement and sizing options
-	 * @param type Type of frame. @sa PageItem::ItemType
-	 * @param locationType Whether the item is only created on the current page or a range 
-	 * @param positionType Location of frame. 0 for 0,0 of page. 1 for 0,0 of page margins, 2, for custom
-	 * @param sizeType Size of frame. 0 for page size, 1 for margin size, 2 for custom
-	 * @param fX X position
-	 * @param fY Y position
-	 * @param fWidth Width of frame
-	 * @param fHeight Height of frame
-	 * @param source Source image or text file for image or text frames
-	 * @param impsetup Import setup for text frames for gtGetText
-	 * @param columnCount Number of columns if created item is a text frame
-	 * @param columnGap Gap between columns if created item is a text frame
-	 * @return 
+	 * @param iafData a InsertAFrameData structure with params
+	 * @return int item id? FIXME
 	 */
 	int itemAddUserFrame(InsertAFrameData &iafData);
 
@@ -584,6 +650,13 @@ public:
 	 * @sa updateFrameItems();
 	 */
 	void renumberItemsInListOrder();
+	/**
+	 * @brief Rebuild item lists taking into account layer order.
+	 * Utility function used in various places, basically handles keeping items numbered in the way
+	 * they are layered. When layer is a property and not a fuction of storage, this should be removed.
+	 * @sa updateFrameItems();
+	 */
+	void rebuildItemLists();
 	/**
 	 * @brief Doc uses automatic text frames?
 	 */
@@ -610,26 +683,39 @@ public:
 	   @param dia optional progress widget
 	 */
 	void RecalcPictures(ProfilesL *Pr, ProfilesL *PrCMYK, QProgressBar *dia = 0);
+	/**
+	 * \brief Handle image with color profiles
+	 * @param items list of page items to update
+	 * @param Pr profile
+	 * @param PrCMYK cmyk profile
+	 * @param dia optional progress widget
+	 */
+	void RecalcPictures(QList<PageItem*>* items, ProfilesL *Pr, ProfilesL *PrCMYK, QProgressBar *dia = 0);
 	/**
 	 * @brief Find the minX,MinY and maxX,maxY for the canvas required for the doc
 	 */
 	void canvasMinMax(FPoint&, FPoint&);
 	
-	int OnPage(double x2, double  y2);
-	int OnPage(PageItem *currItem);
+	int  OnPage(double x2, double  y2);
+	int  OnPage(PageItem *currItem);
 	void GroupOnPage(PageItem *currItem);
-	//void reformPages(double& maxX, double& maxY, bool moveObjects = true);
+
 	void reformPages(bool moveObjects = true);
+	/** @brief Refresh automatic guides once Margin struct has been properly configure by reformPages() */
+	void refreshGuides();
+	/** @brief Check and fix if needed PageItem OwnPage member */
+	void fixItemPageOwner();
 	
 	/**
 	 * @brief Return the x or y offset for a page on the canvas
 	 * @retval double containing the offset. Returns -1.0 if page not in Pages list (as -ve is not possible).
 	 * Mostly saves bringing in extra includes into files that already have scribusdoc.h
 	 */
-	const double getXOffsetForPage(const int);
-	const double getYOffsetForPage(const int);
-	void getBleeds(int pageNumber, double *bleedTop, double *bleedBottom, double *bleedLeft, double *bleedRight);
-	void getBleeds(Page* page, double *bleedTop, double *bleedBottom, double *bleedLeft, double *bleedRight);
+	double getXOffsetForPage(const int);
+	double getYOffsetForPage(const int);
+	void getBleeds(int pageNumber, MarginStruct& bleedData);
+	void getBleeds(const Page* page, MarginStruct& bleedData);
+	void getBleeds(const Page* page, const MarginStruct& baseValues, MarginStruct& bleedData);
 	
 	/**
 	 * @brief Item type conversion functions
@@ -639,7 +725,7 @@ public:
 	/**
 	 * @brief The page number of the current page
 	 */
-	const int currentPageNumber();
+	int currentPageNumber();
 	
 	/**
 	 * @brief Return true iff the passed name is not used by any existing PageItem
@@ -704,8 +790,9 @@ public:
 	void updateAllItemQColors();
 	//! @brief Some internal align tools
 	typedef enum {alignFirst, alignLast, alignPage, alignMargins, alignGuide, alignSelection } AlignTo;
+	typedef enum {alignByMoving, alignByResizing } AlignMethod;
 	void buildAlignItemList(Selection* customSelection=0);
-	bool startAlign();
+	bool startAlign(uint minObjects = 1);
 	void endAlign();
 	/**
 	 * \brief Insert a color into the documents color list
@@ -724,12 +811,22 @@ public:
 	 * @param pageIndex Index of page to find location for
 	 * @return LeftPage, MiddlePage, RightPage, enum from pagestructs.h
 	 */
-	PageLocation locationOfPage(int pageIndex);
+	PageLocation locationOfPage(int pageIndex) const;
+	/**
+	 * \brief Get the column of the page on the canvas, ie, left, middle, or right
+	 * @param pageIndex Index of page to find location for
+	 * @return int of 0,1,2,3
+	 */
+	int columnOfPage(int pageIndex) const;
 	
 	bool sendItemSelectionToBack();
 	bool bringItemSelectionToFront();
 
-	void itemSelection_ApplyParagraphStyle(const ParagraphStyle & newstyle, Selection* customSelection=0);
+	void itemSelection_GroupObjects  (bool changeLock, bool lock, Selection* customSelection=0);
+	void itemSelection_UnGroupObjects(Selection* customSelection=0);
+	void itemSelection_convertItemsTo(const PageItem::ItemType newType, Selection* restoredSelection=0, Selection* customSelection=0);
+
+	void itemSelection_ApplyParagraphStyle(const ParagraphStyle & newstyle, Selection* customSelection=0, bool rmDirectFormatting = false);
 	void itemSelection_SetParagraphStyle(const ParagraphStyle & newstyle, Selection* customSelection=0);
 	void itemSelection_ApplyCharStyle(const CharStyle & newstyle, Selection* customSelection=0);
 	void itemSelection_SetCharStyle(const CharStyle & newstyle, Selection* customSelection=0);
@@ -738,11 +835,12 @@ public:
 
 	void itemSelection_SetNamedParagraphStyle(const QString & name, Selection* customSelection=0);
 	void itemSelection_SetNamedCharStyle(const QString & name, Selection* customSelection=0);
+	void itemSelection_SetNamedLineStyle(const QString & name, Selection* customSelection=0);
 
-	void ChLineWidth(double w);
-	void ChLineArt(Qt::PenStyle w);
-	void ChLineJoin(Qt::PenJoinStyle w);
-	void ChLineEnd(Qt::PenCapStyle w);
+	void itemSelection_SetLineWidth(double w);
+	void itemSelection_SetLineArt(Qt::PenStyle w);
+	void itemSelection_SetLineJoin(Qt::PenJoinStyle w);
+	void itemSelection_SetLineEnd(Qt::PenCapStyle w);
 	void itemSelection_SetAlignment(int w, Selection* customSelection=0);
 	void itemSelection_SetLineSpacing(double w, Selection* customSelection=0);
 	void itemSelection_SetLineSpacingMode(int w, Selection* customSelection=0);
@@ -762,6 +860,9 @@ public:
 	void itemSelection_SetStrikethru(int pos, int wid, Selection* customSelection=0);
 	void itemSelection_SetEffects(int s, Selection* customSelection=0);
 	void itemSelection_SetOpticalMargins(int i, Selection* customSelection=0);
+	void itemSelection_resetOpticalMargins(Selection* customSelection=0);
+	void itemSelection_SetColorProfile(const QString& profileName, Selection* customSelection=0);
+	void itemSelection_SetRenderIntent(int intentIndex, Selection* customSelection=0);
 	
 //	void chAbStyle(PageItem *currItem, int s);
 
@@ -796,7 +897,7 @@ public:
 	/*! \brief Does this doc have any TOC setups and potentially a TOC to generate */
 	bool hasTOCSetup() { return !docToCSetups.empty(); }
 	//! \brief Get the closest guide to the given point
-	void getClosestGuides(double xin, double yin, double *xout, double *yout);
+	void getClosestGuides(double xin, double yin, double *xout, double *yout, int *GxM, int *GyM, Page* pg = NULL);
 	//! \brief Snap an item to the guides
 	void SnapToGuides(PageItem *currItem);
 	bool ApplyGuides(double *x, double *y);
@@ -808,6 +909,16 @@ public:
 	bool SizeItem(double newX, double newY, PageItem *pi, bool fromMP = false, bool DoUpdateClip = true, bool redraw = true);
 	bool MoveSizeItem(FPoint newX, FPoint newY, int ite, bool fromMP = false, bool constrainRotation=false);
 	void AdjustItemSize(PageItem *currItem);
+	void moveGroup(double x, double y, bool fromMP = false, Selection* customSelection = 0);
+	void rotateGroup(double angle, FPoint RCenter);
+	void scaleGroup(double scx, double scy, bool scaleText=true, Selection* customSelection = 0);
+	//! \brief Get a list of frames of certain type
+	QMap<PageItem*, QString> getDocItemNames(PageItem::ItemType itemType);
+
+	//! \brief Fonctions which avoid doc updater and update manager to send too much
+	// unncessary signals when doing updates on multiple items
+	void beginUpdate();
+	void endUpdate();
 	
 protected:
 	void addSymbols();
@@ -837,6 +948,8 @@ public: // Public attributes
 	bool GuideLock;
 	/** \brief Scratch space around Pages */
 	MarginStruct scratch;
+	double GapHorizontal;
+	double GapVertical;
 // 	double ScratchLeft;
 // 	double ScratchRight;
 // 	double ScratchTop;
@@ -847,18 +960,18 @@ public: // Public attributes
 	double rulerXoffset;
 	double rulerYoffset;
 	/** \brief List of Pages */
-	QPtrList<Page>* Pages;
+	QList<Page*>* Pages;
 	/** \brief List of Master Pages */
-	QPtrList<Page> MasterPages;
+	QList<Page*> MasterPages;
 	/** \brief List of Document Pages */
-	QPtrList<Page> DocPages;
+	QList<Page*> DocPages;
 	/** \brief Mapping Master Page Name to Master Page numbers */
 	QMap<QString,int> MasterNames;
 	/** \brief List of Objects */
-	QPtrList<PageItem>* Items;
-	QPtrList<PageItem> MasterItems;
-	QPtrList<PageItem> DocItems;
-	QPtrList<PageItem> FrameItems;
+	QList<PageItem*>* Items;
+	QList<PageItem*> MasterItems;
+	QList<PageItem*> DocItems;
+	QList<PageItem*> FrameItems;
 	Selection* const m_Selection;
 	/** \brief Pagewidth  */
 	double pageWidth;
@@ -868,7 +981,8 @@ public: // Public attributes
 	// int pageCount; Disabled CR no longer required
 	/** \brief Margins */
 	MarginStruct pageMargins;
-	QValueList<PageSet> pageSets;
+	int marginPreset;
+	QList<PageSet> pageSets;
 	MarginStruct bleeds;
 // 	double BleedTop;
 // 	double BleedLeft;
@@ -898,13 +1012,13 @@ public: // Public attributes
 	QString DocName;
 	QMap<QString,int> UsedFonts;
 	SCFonts * const AllFonts;
-	QValueList<AlignObjs> AObjects;
+	QList<AlignObjs> AObjects;
 	QColor papColor;
 	int CurrentSel;
 	ParagraphStyle currentStyle;
 
-	bool EditClip;
-	int EditClipMode;
+	NodeEditContext nodeEdit;
+
 	typoPrefs typographicSettings;
 	guidesPrefs guidesSettings;
 	toolPrefs toolSettings;
@@ -918,36 +1032,37 @@ public: // Public attributes
 	bool leaveDrag;
 	PageItem *DraggedElem;
 	PageItem *ElemToLink;
-	QValueList<uint> DragElements;
+	QList<uint> DragElements;
 private:
 	StyleSet<ParagraphStyle> docParagraphStyles;
 	StyleSet<CharStyle> docCharStyles;
 public:
-	QValueList<Layer> Layers;
+	ScLayers Layers;
 	bool marginColored;
 	int GroupCounter;
 	CMSData CMSSettings;
-	cmsHPROFILE DocInputImageRGBProf;
-	cmsHPROFILE DocInputImageCMYKProf;
-	cmsHPROFILE DocInputRGBProf;
-	cmsHPROFILE DocInputCMYKProf;
-	cmsHPROFILE DocOutputProf;
-	cmsHPROFILE DocPrinterProf;
-	cmsHTRANSFORM stdTransRGBMon;
-	cmsHTRANSFORM stdTransCMYKMon;
-	cmsHTRANSFORM stdProof;
-	cmsHTRANSFORM stdTransImg;
-	cmsHTRANSFORM stdProofImg;
-	cmsHTRANSFORM stdProofImgCMYK;
-	cmsHTRANSFORM stdTransCMYK;
-	cmsHTRANSFORM stdProofCMYK;
-	cmsHTRANSFORM stdTransRGB;
-	cmsHTRANSFORM stdProofGC;
-	cmsHTRANSFORM stdProofCMYKGC;
+	ScColorProfile DocInputImageRGBProf;
+	ScColorProfile DocInputImageCMYKProf;
+	ScColorProfile DocInputRGBProf;
+	ScColorProfile DocInputCMYKProf;
+	ScColorProfile DocOutputProf;
+	ScColorProfile DocPrinterProf;
+	ScColorTransform stdTransRGBMon;
+	ScColorTransform stdTransCMYKMon;
+	ScColorTransform stdProof;
+	ScColorTransform stdTransImg;
+	ScColorTransform stdProofImg;
+	ScColorTransform stdProofImgCMYK;
+	ScColorTransform stdTransCMYK;
+	ScColorTransform stdProofCMYK;
+	ScColorTransform stdTransRGB;
+	ScColorTransform stdProofGC;
+	ScColorTransform stdProofCMYKGC;
+	bool BlackPoint;
 	bool SoftProofing;
 	bool Gamut;
-	int IntentColors;
-	int IntentImages;
+	eRenderIntent IntentColors;
+	eRenderIntent IntentImages;
 	bool HasCMS;
 	QMap<QString,QString> JavaScripts;
 	int TotalItems;
@@ -956,7 +1071,6 @@ public:
 	QString Language;
 	bool Automatic;
 	bool AutoCheck;
-	
 	PDFOptions PDF_Options;
 	PrintOptions Print_Options;
 	bool RePos;
@@ -972,7 +1086,7 @@ public:
 					int Prev;
 					int Next;
 					};
-	QValueList<BookMa> BookMarks;
+	QList<BookMa> BookMarks;
 	bool OldBM;
 	bool hasName;
 	int RotMode;
@@ -980,7 +1094,7 @@ public:
 	int AutoSaveTime;
 	QTimer * const autoSaveTimer;
 	QMap<QString,multiLine> MLineStyles;
-	QValueList<ArrowDesc> arrowStyles;
+	QList<ArrowDesc> arrowStyles;
 	QMap<QString, ScPattern> docPatterns;
 	QWidget* WinHan;
 	bool DoDrawing;
@@ -990,7 +1104,7 @@ public:
 		Page *page;
 		PageItem *item;
 	};
-	QValueList<OpenNodesList> OpenNodes;
+	QList<OpenNodesList> OpenNodes;
 	QTimer *CurTimer;
 	QMap<int, errorCodes> docLayerErrors;
 	QMap<int, errorCodes> docItemErrors;
@@ -1008,8 +1122,15 @@ public:
 	
 	Hyphenator * docHyphenator;
 private:
-	bool _itemCreationTransactionStarted;
-	Page* _currentPage;
+	UndoTransaction* m_itemCreationTransaction;
+	UndoTransaction* m_alignTransaction;
+
+	Page* m_currentPage;
+	UpdateManager m_updateManager;
+	MassObservable<PageItem*> m_itemsChanged;
+	MassObservable<Page*> m_pagesChanged;
+	MassObservable<QRectF> m_regionsChanged;
+	DocUpdater* m_docUpdater;
 	
 signals:
 	//Lets make our doc talk to our GUI rather than confusing all our normal stuff
@@ -1042,15 +1163,23 @@ public slots:
 	void itemSelection_ToggleImageShown();
 	void itemSelection_TogglePrintEnabled();
 	void itemSelection_ChangePreviewResolution(int id);
+
+	/*! \brief Change display quality of all images in document.
+	\author  OssiLehtinen
+	*/
+	void allItems_ChangePreviewResolution(int id);
+
 	void itemSelection_ClearItem(Selection* customSelection=0);
 	//! Delete the items in the current selection. When force is true, we do not warn the user and make SE happy too. Force is used from @sa Page::restorePageItemCreation
 	void itemSelection_DeleteItem(Selection* customSelection=0, bool forceDeletion=false);
+	void itemSelection_SetItemTextReversed(bool reversed);
 	void itemSelection_SetItemFillTransparency(double t);
 	void itemSelection_SetItemLineTransparency(double t);
 	void itemSelection_SetItemFillBlend(int t);
 	void itemSelection_SetItemLineBlend(int t);
 	void itemSelection_SetLineGradient(VGradient& newGradient, Selection* customSelection=0);
 	void itemSelection_SetFillGradient(VGradient& newGradient, Selection* customSelection=0);
+	void itemSelection_SetOverprint(bool overprint, Selection* customSelection=0);
 	void itemSelection_ApplyImageEffects(ScImageEffectList& newEffectList, Selection* customSelection=0);
 	void itemSelection_FlipH();
 	void itemSelection_FlipV();
@@ -1060,22 +1189,26 @@ public slots:
 	void itemSelection_SetImageOffset(double x, double y, Selection* customSelection=0);
 	void itemSelection_SetImageScale(double x, double y, Selection* customSelection=0);
 	void itemSelection_SetImageScaleAndOffset(double ox, double oy, double sx, double sy, Selection* customSelection=0);
-	void itemSelection_AlignLeftOut(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignRightOut(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignBottomIn(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignRightIn(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignBottomOut(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignCenterHor(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignLeftIn(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignCenterVer(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignTopOut(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_AlignTopIn(AlignTo currAlignTo, double guidePosition);
-	void itemSelection_DistributeDistH(bool usingDistance=false, double distance=0.0);
+	void itemSelection_AlignItemLeft(int i, double newX, AlignMethod how);
+	void itemSelection_AlignItemRight(int i, double newX, AlignMethod how);
+	void itemSelection_AlignItemTop(int i, double newY, AlignMethod how);
+	void itemSelection_AlignItemBottom(int i, double newY, AlignMethod how);
+	void itemSelection_AlignLeftOut(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignRightOut(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignBottomIn(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignRightIn(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignBottomOut(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignCenterHor(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignLeftIn(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignCenterVer(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignTopOut(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_AlignTopIn(AlignTo currAlignTo, AlignMethod currAlignMethod, double guidePosition);
+	void itemSelection_DistributeDistH(bool usingDistance=false, double distance=0.0, bool reverseDistribute=false);
 	void itemSelection_DistributeAcrossPage(bool useMargins=false);
 	void itemSelection_DistributeRight();
 	void itemSelection_DistributeBottom();
 	void itemSelection_DistributeCenterH();
-	void itemSelection_DistributeDistV(bool usingDistance=false, double distance=0.0);
+	void itemSelection_DistributeDistV(bool usingDistance=false, double distance=0.0, bool reverseDistribute=false);
 	void itemSelection_DistributeDownPage(bool useMargins=false);
 	void itemSelection_DistributeLeft();
 	void itemSelection_DistributeCenterV();
@@ -1086,21 +1219,31 @@ public slots:
 	/**
 	 * Adjust an image frame's size to fit the size of the image in it
 	 */
-	void itemSelection_adjustFrametoImageSize(Selection* customSelection=0);
+	void itemSelection_AdjustFrametoImageSize(Selection* customSelection=0);
+	/**
+	 * Adjust an image size to fit the size of the frame
+	 */
+	void itemSelection_AdjustImagetoFrameSize(Selection* customSelection=0);
 	//! @brief startArrowID or endArrowID of -1 mean not applying a selection at this point.
 	void itemSelection_ApplyArrowHead(int startArrowID=-1, int endArrowID=-1, Selection* customSelection=0);
 
-	void ItemPen(QString farbe);
-	void ItemPenShade(int sha);
-	void ItemBrush(QString farbe);
-	void ItemBrushShade(int sha);
-	void ItemGradFill(int typ);
-	void ItemPatternFill(QString pattern);
-	void ItemPatternProps(double scaleX, double scaleY, double offsetX, double offsetY, double rotation);
+	void itemSelection_SetItemPen(QString farbe);
+	void itemSelection_SetItemPenShade(int sha);
+	void itemSelection_SetItemBrush(QString farbe);
+	void itemSelection_SetItemBrushShade(int sha);
+	void itemSelection_SetItemGradFill(int typ);
+	void itemSelection_SetItemPatternFill(QString pattern);
+	void itemSelection_SetItemPatternProps(double scaleX, double scaleY, double offsetX, double offsetY, double rotation);
+
+	void undoRedoBegin();
+	void undoRedoDone();
 
 	void updatePic();
 	void updatePict(QString name);
+	void updatePictDir(QString name);
 	void removePict(QString name);
 };
 
+Q_DECLARE_METATYPE(ScribusDoc*);
+
 #endif
